import 'dart:collection';
import 'dart:typed_data';

import '../../omp_client.dart';

class BufferOutputStream {
  final _buffer;

  BufferOutputStream(this._buffer);

  void clearBuffer() {
    if (_buffer is List<int>) {
      (_buffer as List<int>).clear();
    } else if (_buffer is Queue<int>){
      (_buffer as Queue<int>).clear();
    }
  }

  void writeRawBytes(List<int> bytes) {
    _buffer.addAll(bytes);
  }

  void writeBytes(List<int> bytes) {
    writeInt(bytes.length);
    _buffer.addAll(bytes);
  }

  void writeCommand(Command cmd) {
    writeString(cmd.toString());
  }

  void writeString(String str) {
    final lastCharIndex = str.length - 1;

    if (str.codeUnitAt(lastCharIndex) == 10)
      str = str.substring(0, lastCharIndex);
    //print("ToSend: ${str}");
    //print("IntCodeUnits: ${Uint8List.fromList(str.codeUnits).buffer.asInt8List()}");
    writeBytes(str.codeUnits);
  }

  void writeBoolean(bool b) {
    _buffer.add(b ? 1 : 0);
  }

  void writeInt(int value) {
    _buffer.add(value >> 24 & 0xFF);
    _buffer.add(value >> 16 & 0xFF);
    _buffer.add(value >> 8 & 0xFF);
    _buffer.add(value >> 0 & 0xFF);
  }
}